//
//  SarpStore.m
//  Cube2Cube
//
//  Created by Zach Stegall on 9/12/13.
//  Copyright (c) 2013 Zach Stegall. All rights reserved.
//

#import "SarpStore.h"
#import "SARP.h"

@implementation SarpStore

+ (SarpStore *)sharedStore
{
    static SarpStore *sharedStore = nil;
    if (!sharedStore) {
        sharedStore = [[super allocWithZone:nil] init];
    }
    
    return sharedStore;
}

+ (id)allocWithZone:(NSZone *)zone
{
    return [self sharedStore];
}

- (id)init
{
    self = [super init];
    
    if (self) {
        model = [NSManagedObjectModel mergedModelFromBundles:nil];
        
        NSPersistentStoreCoordinator *psc = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:model];
        
        NSString *path = [self itemsArchivePath];
        NSURL *sarpURL = [NSURL fileURLWithPath:path];
        
        NSError *error = nil;
        
        if (![psc addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:sarpURL options:nil error:&error]) {
            [NSException raise:@"Open failed" format:@"Reason: %@", [error localizedDescription]];
        }
        
        context = [[NSManagedObjectContext alloc] init];
        [context setPersistentStoreCoordinator:psc];
        [context setUndoManager:nil];
        
        [self loadAllItems];
    }
    
    return self;
}

- (NSString *)itemsArchivePath
{
    NSArray *documentDirectories = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentDirectory = [documentDirectories objectAtIndex:0];
    
    return [documentDirectory stringByAppendingPathComponent:@"sarp.sqlite"];
}

- (BOOL)saveChanges
{
    NSError *err = nil;
    BOOL successfull = [context save:&err];
    if (!successfull) {
        NSLog(@"Error saving: %@", [err localizedDescription]);
    }
    
    return successfull;
}

- (void)loadAllItems
{
    if (!allItems) {
        
    }
    
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entitydesc = [[model entitiesByName] objectForKey:@"SARP"];
    [fetchRequest setEntity:entitydesc];
    
    NSSortDescriptor *sd = [NSSortDescriptor sortDescriptorWithKey:@"firstName" ascending:YES];
    [fetchRequest setSortDescriptors:[NSArray arrayWithObject:sd]];
    
    NSError *fetchError;
    NSArray *matchingData = [context executeFetchRequest:fetchRequest error:&fetchError];
    
    if(!matchingData) {
        [NSException raise:@"Fetch failed" format:@"Reason: %@", [fetchError localizedDescription]];
    }
    
    allItems = [[NSMutableArray alloc] initWithArray:matchingData];
}

- (NSArray *)allItems
{
    return allItems;
}

- (SARP *)createItem:(NSString *)first lastName:(NSString *)last
{
    SARP *p = [NSEntityDescription insertNewObjectForEntityForName:@"SARP" inManagedObjectContext:context];
    
    [p setFirstName:first];
    [p setLastName:last];
    [allItems addObject:p];
    
    [self saveChanges];
    
    return p;
}

- (void)addExperience:(NSString *)experience firstname:(NSString *)first lastName:(NSString *)last
{
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"SARP" inManagedObjectContext:context];
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"(firstName like %@) and (lastName like %@)", first, last];
    [request setEntity:entity];
    [request setPredicate:predicate];
    
    NSError *error;
    [context executeFetchRequest:request error:&error];

    [self saveChanges];
}

@end
